#uses "fwUnitTestComponentAsserts.ctl"

#uses "test/fwRPC/utils.ctl"

#uses "fwRPC/fwRPC_common.ctl"
#uses "fwRPC/fwRPC_functionModel.ctl"
#uses "fwRPC/fwRPC_functionManager.ctl"
#uses "fwRPC/fwRPC_server.ctl"


// Mocks / utils
//------------------------------------------------------------------------------

const string funcName = "test_func";


// Setup
//------------------------------------------------------------------------------

void test_fwRPC_functionManager_setupSuite() {
  _BEGIN_SUITE();
}

void test_fwRPC_functionManager_teardownSuite() {
  _setup_clearAll();
  _END_SUITE();
}

void test_fwRPC_functionManager_setup() {
  _BEGIN_TEST();
  _setup_clearAll();
}

void _setup_clearAll() {
  string dp = fwRPC_common_makeDpName(fwRPC_FUNCTION_DPT, funcName);
  if (dpExists(dp)) {
    dpDelete(dp);
  }

  fwRPC_functionManager_clearRegistry();
}


// Tests
//------------------------------------------------------------------------------

void test_fwRPC_functionManager_register() {
  dyn_mixed funcObject = fwRPC_functionModel_new(funcName);

  assertFalse(fwRPC_functionManager_isRegistered(funcName));

  fwRPC_functionManager_register(funcObject);

  assertTrue(fwRPC_functionManager_isRegistered(funcName));
}


void test_fwRPC_functionManager_get() {
  dyn_mixed funcObject = fwRPC_functionModel_new(funcName);
  dyn_mixed newObject;

  fwRPC_functionManager_register(funcObject);

  fwRPC_functionManager_getFunctionObject(funcName, newObject);

  assertEqual(funcObject, newObject);
}


void test_fwRPC_functionManager_errOn_getNonExist() {
  bool wasThrown = false;

  try {
    fwRPC_functionManager_getFunctionObject(funcName);
  } catch {
    wasThrown = true;
  }

  assertTrue(wasThrown);
}


void test_fwRPC_functionManager_errOn_duplicate() {
  dyn_mixed funcObject = fwRPC_functionModel_new(funcName);
  bool wasThrown = false;

  fwRPC_functionManager_register(funcObject);
  try {
    fwRPC_functionManager_register(funcObject);
  } catch {
    wasThrown = true;
  }

  assertTrue(wasThrown);
}


void test_fwRPC_functionManager_clearAll() {
  string funcName2 = funcName + "2";

  dyn_mixed funcObject = fwRPC_functionModel_new(funcName);
  dyn_mixed funcObject2 = fwRPC_functionModel_new(funcName2);

  fwRPC_functionManager_register(funcObject);
  fwRPC_functionManager_register(funcObject2);
  fwRPC_functionManager_clearRegistry();

  assertFalse(fwRPC_functionManager_isRegistered(funcName));
  assertFalse(fwRPC_functionManager_isRegistered(funcName2));
}


void test_fwRPC_functionManager_loadFromServerReg() {
  fwRPC_registerFunction(funcName, "", "");

  fwRPC_functionManager_load();

  assertTrue(fwRPC_functionManager_isRegistered(funcName));
}
