/**@name LIBRARY: unSystemIntegrity_CMWClient.ctl

@author: Celine VUILLERMOZ (AB-CO)

Creation Date: 07/10/2005

Modification History: 
  03/10/2011: Herve
  - IS-600  unLHCServices  system name and component version added in the MessageText log and in the diagnostic 
 
  16/06/2011: Herve
  - IS-556: get the list of systemAlarm pattern
  
  30/05/2008: Herve
    - add check on DriverCommon.OV, ST and CMW_driver.cmw_QueueOverflow
    	
version 1.0

External Functions: 
	
Internal Functions: 
	
Purpose: 
	Library of the CMWClient component used by the systemIntegrity.
	 	
Usage: Public

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. global variable: the following variables are global to the script
	. data point type needed: _UnSystemIntegrity
	. data point: CMWClient_systemIntegrity
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------
const string UN_SYSTEM_INTEGRITY_CMWClient = "CMWClient";
const string c_unSystemIntegrity_CMWClient_callback = "_unSystemIntegrity_CMWClientCallback";
const int UNKNOWN_CMWCLIENT_STATE = -1;
// 
//------------------

// global declaration
global dyn_string g_unSystemIntegrity_CMWClientList; // list of the CMWClient dp checked
//CMWClient

dyn_string g_dsCMWClient_Thread;					//Thread started
global int g_unSystemIntegrity_CMWClientCheckingDelay;
int c_unSystemIntegrity_defaultCMWClientCheckDelay =15;
global mapping g_unSystemIntegrity_CMWClientState;

//@{

//------------------------------------------------------------------------------------------------------------------------
// CMWClient_systemIntegrityInfo
/** Return the list of systemAlarm pattern. 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@return list of systemAlarm pattern
*/
dyn_string CMWClient_systemIntegrityInfo()
{
  return makeDynString(CMWClient_pattern);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_CMWClient_Initialize
/**
Purpose:
Get the list of CMWClient check by the systemIntegrity. The ones that are enabled.

@param dsResult: dyn_string, output, the list of enabled CMWClient that are checked

Usage: Public

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_CMWClient_Initialize(dyn_string &dsResult)
{
	dyn_string dsList;
	int len, i;
	string dpToCheck;
	bool enabled;
  string sMessage;
  dyn_string exceptionInfo;
  
  sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("unCMWClient"));
	
	dpGet(UN_SYSTEM_INTEGRITY_DPE+UN_SYSTEMINTEGRITY_EXTENSION+".config.data", dsList);
	unSystemIntegrity_DPE_DataCallback("", dsList);
		
// get the list of CMWClient manager dp to checked
	dsList = dpNames(c_unSystemAlarm_dpPattern+CMWClient_pattern+"*", c_unSystemAlarm_dpType);
	len = dynlen(dsList);
	for(i = 1; i<=len; i++) {
		dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
		dpToCheck = substr(dpToCheck, strpos(dpToCheck,CMWClient_pattern));
		dpGet(dsList[i]+".enabled", enabled);
		if(enabled)
			dynAppend(dsResult, dpToCheck);
	}
	dpConnect("unSystemIntegrity_CMWClient_DataCallback", UN_SYSTEM_INTEGRITY_CMWClient+UN_SYSTEMINTEGRITY_EXTENSION+".config.data");

  unMessageText_send("*", "*", g_SystemIntegrity_sSystemName+"systemIntegrity", "user", "*", "INFO", "systemIntegrity "+sMessage+" loaded", exceptionInfo);
//DebugN(dsResult);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_CMWClient_HandleCommand
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_CMWClient_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len =dynlen(parameters);
  string sMessage;
	//DebugN(parameters);
	
	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_CMWClient_checking(parameters[i], true, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_CMWClient_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+parameters[i]);
			}
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_CMWClient_checking(parameters[i], false, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_CMWClient_checking(parameters[i], false, false, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSet(UN_SYSTEM_INTEGRITY_CMWClient+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
					UN_SYSTEM_INTEGRITY_CMWClient+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_unSystemIntegrity_CMWClientList);
			break;
    case UN_SYSTEMINTEGRITY_VERSION:
      sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("unCMWClient"));
      dpSet(UN_SYSTEM_INTEGRITY_CMWClient+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
            UN_SYSTEM_INTEGRITY_CMWClient+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", makeDynString(getCurrentTime(),sMessage));
      break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
			unMessageText_sendException("*", "*", "unSystemIntegrity_CMWClient_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error uin case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugN(getCurrentTime(), exceptionInfo);
		}
	}
//	DebugN(sDpe1, command, parameters, g_unSystemIntegrity_CMWClientList);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_CMWClient_checking
/**
Purpose:
This function register/de-register the callback funtion of CMWClient manager. This function can also create the _unSystemAlarm_CMWClient dp 
with the alarm config but it cannot delete it.

	@param sDp_Num: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bRegister: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfoTemp: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

unSystemIntegrity_CMWClient_checking(string sDp_Num, bool bCreate, bool bRegister, dyn_string &exceptionInfoTemp)
{
string sSystem, dp_Alarm, description, sManager, dpeToCheck_1, dpeToCheck_11, dpeToCheck_10, sNum;
int iIndex, iRes, iCMWClient_Num, iThread, len, j;
bool found = false;

//DebugN("LHCCMWClient", "_unSystemIntegrity_set_CMWClient_checking");
	sSystem=getSystemName();
	//g_bPanel=false;// IS-1745, not needed anymore; to be removed
	
	//1. Get Alarm Name
	dp_Alarm = c_unSystemAlarm_dpPattern + sDp_Num;
	
	//2. Get Manager Name
	sDp_Num=substr(sDp_Num,strpos(sDp_Num,"_")+1);
	if (strlen(sDp_Num)<2)
		sNum="0" + sDp_Num;
	else
		sNum=sDp_Num;
	
	sManager=CMWClient_pattern + "Driver_" + sNum;
	
	//3. Get data...
//	dpeToCheck_1 = sSystem + sManager + ".drv_Num";

	if (bRegister)
		{
// check if the dp_alarm is in the global list if not then dpConnect, it is the first time, 
// if yes then it is a second time do not twice dpConnect
// this global variable is set in the _unSystemIntegrity_registerFunction
		iRes = _unSystemIntegrity_isInList(g_unSystemIntegrity_CMWClientList, dp_Alarm);
		if(iRes <= 0 && dpExists(sSystem + sManager)) 
			{
				//Create the dp_Alarm if it is not existing
				if(!dpExists(dp_Alarm))
					{
					description = getCatStr("unSystemIntegrity", "CMWClient_DESCRIPTION") + sDp_Num;
					unSystemIntegrity_createSystemAlarm(sDp_Num, CMWClient_pattern, description, exceptionInfoTemp);
					}
				
				//Connect
				if(dynlen(exceptionInfoTemp)<=0)
					{
                                        _unSystemIntegrity_CMWClient_setState(dpSubStr(sManager, DPSUB_SYS_DP), UNKNOWN_CMWCLIENT_STATE);
					//Start Thread to check Period 
					iThread=startThread("unCMWClient_Integrity", dp_Alarm, dpSubStr(sManager, DPSUB_SYS_DP));
					
					if (iThread>=0)
						dynAppend(g_dsCMWClient_Thread, sManager + "." + iThread);		
						_unSystemIntegrity_registerFunction(bRegister, "unCMWClient_Integrity", makeDynString(dp_Alarm,sManager), 
							g_unSystemIntegrity_CMWClientList, dp_Alarm, exceptionInfoTemp);
							
                                                dpConnect("unSystemIntegrity_CMWClient_CB", "_Driver"+sDp_Num+".OV", 
                                                          "_Driver"+sDp_Num+".DT", sManager+".cmw_QueueOverflow");
					}
			}
		}
	else
		{
// check if the dp_alarm is in the global list if not then do nothing, 
// if yes then d a dpDisconnect
// this global variable is set in the _unSystemIntegrity_Logging_SetArchiveCheck
		iRes = _unSystemIntegrity_isInList(g_unSystemIntegrity_CMWClientList, dp_Alarm);
		if(iRes > 0) 
			{

			//Stop Thread
			len=dynlen(g_dsCMWClient_Thread);
			for (j=1;(j<=len) && (found == false);j++)		
				{
				if (patternMatch(sManager + "*", g_dsCMWClient_Thread[j]))
					{			
					iThread=substr(g_dsCMWClient_Thread[j],strpos(g_dsCMWClient_Thread[j],".")+1,strlen(g_dsCMWClient_Thread[j]));
					if (iThread>0)
						{
						found = true;
						iIndex = j;
//					j=len+1;	//exit loop
						}
					}
				}
			
			if (found)
				{
				iRes=stopThread(iThread);
			
				if (iRes==0)
					dynRemove(g_dsCMWClient_Thread,iIndex);
				}
		
		_unSystemIntegrity_registerFunction(bRegister, "unCMWClient_Integrity", makeDynString(dp_Alarm,sManager), 
						g_unSystemIntegrity_CMWClientList, dp_Alarm, exceptionInfoTemp);
                 dpDisconnect("unSystemIntegrity_CMWClient_CB", "_Driver"+sDp_Num+".OV", 
                                            "_Driver"+sDp_Num+".DT", sManager+".cmw_QueueOverflow");

			}
                         _unSystemIntegrity_CMWClient_setState(dpSubStr(sManager, DPSUB_SYS_DP), UNKNOWN_CMWCLIENT_STATE);		
                     }
	
}

////////////////////

// unCMWClient_Integrity
/**
Purpose:
Function to check the value of a DPE. 
Set an alarm if the value has not changed and if the time has not changed and the stime_inv bit is set to false.
in the other case reset it.

	@param sAlarmDp: string, input, alarm dp
	@param dpToCheck: string, input, dpe to check

Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

unCMWClient_Integrity(string sAlarmDp, string sDpToCheck)
{
	int waitingTime;
	int oldValue = -1, newValue, alarmValue, oldAlarmValue = -1;
//	time oldTime=0, newTime;
	bool bStimeInv;
	bool bEnabled;
  int iCurrentState;
  
/*	if(sDpToCheck <10)	sDpToCheck = "CMWClient_Driver_0"+ sDpToCheck;
	else sDpToCheck = "CMWClient_Driver_"+ sDpToCheck;*/
//DebugN("g_unSystemIntegrity_CMWClientCheckingDelay, sAlarmDp, sDpToCheck",g_unSystemIntegrity_CMWClientCheckingDelay, sAlarmDp, sDpToCheck);
	// get the value of the DPE and the old time
	dpGet(sDpToCheck+".drv_Num:_online.._stime", oldValue);
	while(true) {
	// if 0 set it to c_unSystemIntegrity_defaultDPECheckDelay
		waitingTime = g_unSystemIntegrity_CMWClientCheckingDelay;
		//DebugN("waitingTime",waitingTime,g_unSystemIntegrity_CMWClientCheckingDelay);
		if(waitingTime <= 0)
			waitingTime = c_unSystemIntegrity_defaultCMWClientCheckDelay;
	// wait
		delay(waitingTime);
	
	// get the value of the counter and the timeout PLCChecking
		dpGet(sDpToCheck+".drv_Num:_online.._stime", newValue/*, sDpToCheck+".:_online.._stime", newTime*/, sDpToCheck+".drv_Num:_online.._stime_inv", bStimeInv);

//		if( bStimeInv || ((newValue == oldValue) && (newTime == oldTime)) ){
		if( bStimeInv || (newValue == oldValue) ){
	// counter was not modified, set an alarm
			alarmValue = c_unSystemIntegrity_alarm_value_level1;
		}
		else {	
	// counter was modified, reset the alarm
			alarmValue = c_unSystemIntegrity_no_alarm_value;
		}
                _unSystemIntegrity_CMWClient_getState(sDpToCheck, iCurrentState);
                if(alarmValue < iCurrentState)
                  alarmValue = iCurrentState;
		if(alarmValue != oldAlarmValue)
			dpSet(sAlarmDp+".alarm", alarmValue);

//DebugTN("unSystemIntegrity_DPECheck (cu, old ==> res", sDpToCheck, iCurrentState, oldAlarmValue, alarmValue);	
		oldValue = newValue;
		oldAlarmValue = alarmValue;
//		oldTime = newTime;
	}
}

// unSystemIntegrity_CMWClient_DataCallback
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param dsConfigData: dyn_string, input, the config data

Usage: Public

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_CMWClient_DataCallback(string sDpe1, dyn_string dsConfigData)
{
	int CMWClientCheckingDelay;
		
	if(dynlen(dsConfigData) > 0)
		g_unSystemIntegrity_CMWClientCheckingDelay = dsConfigData[1];

	if(g_unSystemIntegrity_CMWClientCheckingDelay <= 0)
		g_unSystemIntegrity_CMWClientCheckingDelay = c_unSystemIntegrity_defaultCMWClientCheckDelay;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity__CMWClient_CB_Driver
/**
Purpose:
Get Api Connections

	@param sDp_1: string, input, data point name
	@param diDriver: dyn_int, Driver Connections
Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unSystemIntegrity__CMWClient_CB_Driver(string sDp_1, dyn_int diDriver)
{
	g_diCMWClient_Driver=diDriver;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_CMWClient_CB
/**
Purpose:
Get Api Connections

	@param sDp_1: string, input, data point name
	@param bOV: bool, driver overflow state
	@param sDp_2: string, input, data point name
	@param sDT: string, driver type name
	@param sDp_3: string, input, data point name
	@param bcmw_QueueOverflow: bool, cmw overflow state
        
Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unSystemIntegrity_CMWClient_CB(string sDp_1, bool bOV, string sDp_2, string sDT, 
                               string sDp_3, bool bcmw_QueueOverflow)
{
  int iState;
//DebugTN(bOV, sDT, bcmw_QueueOverflow, dpSubStr(sDp_3, DPSUB_SYS_DP));

  if(bOV)
    iState = 2*c_unSystemIntegrity_alarm_value_level1;
  else if(sDT != "CMW")
    iState = 3*c_unSystemIntegrity_alarm_value_level1;
  else if(bcmw_QueueOverflow)
    iState = 4*c_unSystemIntegrity_alarm_value_level1;
  else
    iState = c_unSystemIntegrity_no_alarm_value;
    
  _unSystemIntegrity_CMWClient_setState(dpSubStr(sDp_3, DPSUB_SYS_DP), iState);
}

//------------------------------------------------------------------------------------------------------------------------

// _unSystemIntegrity_CMWClient_setState
/**
Purpose:
Set state

	@param sManager: string, input, key
	@param iState: int, state to set
        
Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_CMWClient_setState(string sManager, int iState) synchronized(g_unSystemIntegrity_CMWClientState)
{
  int iCurrentState;
  
  if(mappingHasKey(g_unSystemIntegrity_CMWClientState, sManager))
    iCurrentState = g_unSystemIntegrity_CMWClientState[sManager];
  else
    iCurrentState = UNKNOWN_CMWCLIENT_STATE;
  g_unSystemIntegrity_CMWClientState[sManager] = iState;
//DebugTN(sManager, iState, g_unSystemIntegrity_CMWClientState);
}

//------------------------------------------------------------------------------------------------------------------------

// _unSystemIntegrity_CMWClient_getState
/**
Purpose:
Get State

	@param sManager: string, input, key
	@param iState: out, state to set
        
Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_CMWClient_getState(string sManager, int &iState) synchronized(g_unSystemIntegrity_CMWClientState)
{
  int iCurrentState;
  
  if(mappingHasKey(g_unSystemIntegrity_CMWClientState, sManager))
    iCurrentState = g_unSystemIntegrity_CMWClientState[sManager];
  else
    iCurrentState = UNKNOWN_CMWCLIENT_STATE;
  iState = iCurrentState;
}

//------------------------------------------------------------------------------------------------------------------------

//@}
